########################################################################################
## Kimberlite - Rift association model uncertainty and migration rate estimates
## Using two end member cases: association with the shortest distance, or shortest time
## here we use Kimberlite records for Africa, South America and North America
##
## Figures generated: 
## (i) 	Fig 1e Density plot of migration rate estimates, accounting for model  
## 			and time/distance uncertainty, for Africa, South America and 
## 			North America (grouped my location)
## (ii) 	Extended data Fig 4a and 4b density plot of migration rate, grouped by 
## 			association model (minimum lag or minimum distance) for: 
##			(a) clusters and (b) individual kimberlites
##			for all three locations (Africa, S America, N America)
## (iii)	Extended data Fig 5a and 5b histograms for N America Kimberlite Clusters 
##			Histogram of time lag relative to breakup using (a) original data, 
##			and (b) incorporating uncertainty
##
## Inputs:
## GIS shapefiles for the individual rift sections: 
##		COB_NAmerica.shp, COB_SAmerica.shp, COB_Africa.shp
## These comprise line segments with several attributes. Here we are interested in 
## (i) 	"FROMAGE" the estimated breakup age
## (ii) 	"RiftAge" estimated age of rifting
## The original shapefiles were generated by GPLATES, see www.gplates.org and
## Muller et al., 2018, https://doi.org/10.1029/2018GC007584
## Rift ages are from Muller et al., 2019, https://doi.org/10.1029/2018TC005462
## 
## Kimberlite data:
## 		Combined_database_Kimberlites_Tappe_Brazil_Uncertainty.shp
## 		Combined_database_Kimberlites_Tappe_Brazil.shp (without age uncertainty)
## From Tappe et al., 2018 https://doi.org/10.1016/j.epsl.2017.12.013
## Age uncertainties are an unpublished compilation of published data provided by
## Sebastian Tappe, who can be contacted for full details of the compilation.
##
## Additional data for Brazil (total 8 Kimberlites) from:
##	Guarino et al., 2013, https://doi.org/10.1016/j.chemgeo.2012.06.016
##	Felgate M., 2014 (PhD Thesis) The petrogenesis of Brazilian kimberlites and 
##		kamafugites intruded along the 125° lineament: improved geochemical and 
##		geochronological constraints on magmatism in Rondonia and the Alto Paranaiba
##		Igneous Province
##	Conceição et al., 2019, https://doi.org/10.1016/j.lithos.2019.01.012
##	Fernandes et al., 2021, https://doi.org/10.1016/j.tecto.2021.229011
##	Carvalho et al, 2022, https://doi.org/10.1016/j.gsf.2021.101281
##	
########################################################################################

mydir="YOUR PATH HERE"
setwd(paste0(mydir,"Generate_migration_rate_estimates"))

require("ggplot2")  
require("viridis") 
require(raster)             
require(geosphere) ## to calculate distances between points and line segments 
require(sp)
require(sf)
require(RobustLinearReg)


# NOTES
# the file Combined_database_Kimberlites_Tappe_Brazil.shp includes all kimberlite records from Tappe et al. 2018
# together with records for eight additional kimberlites in Brazil (see Methods section)
#
# the file Combined_database_Kimberlites_Tappe_Brazil_Uncertainty.shp additionally contains kimberlite age uncertainties 
# these uncertainty estimates are published data, from an unpublished compilation provided by Sebastian Tappe. 
#
# "Location" is the broad geographic location:
# [1] "Africa"     "Australia"  "E_Europe"   "N_America"  "E_Eurasia"  "S_America"  "Greenland" 
# [8] NA           "Antarctica"
# For simplicity, Kimberlites with Location = N_America includes Canada and Greenland
# Sites with missing LON/LAT (NA in database) are excluded  from the analysis
# [there are only 14 sites with missing LON/LAT]
# "Source" gives the data source which is either "Tappe" or "Brazil"

### Note this is the file with uncertainty estimates:
AllKimberlites <- st_read(dsn = "../GIS", layer = "Combined_database_Kimberlites_Tappe_Brazil_Uncertainty")

# only keep kimberlites with valid lon/lat:
AllKimberlites=AllKimberlites[which(!is.na(AllKimberlites$Longitude)&!is.na(AllKimberlites$Latitude)),]


list_ids=list()

### exclude Kimberlites much older than rifting 
# (oldest rift age is ~240Ma, so cutoff at 270 to be conservative)

AllKimberlites=AllKimberlites[which(AllKimberlites$Age<270),]
# total of 731 records

# give the remaining kimberlites a unique ID field
AllKimberlites$fid<-1:nrow(AllKimberlites)

# These are the ridge/rift locations (and associated shapefiles):
Locations=c("N_America","S_America","Africa")

## Rift shapefiles:
Rifts=c("COB_NAmerica","COB_SAmerica","COB_Africa")


## define some structures to store sampled ages/lags/distances
## use a list with one object per location
## for each location we have a (sub) list of sampled values, one object per ridge section


## 
## order in time (i.e start with oldest rift section)
## attribute using rift age, limiting rate (dist/lag rel to rift initiation)

list_lag_breakup=list()
list_dist_breakup=list()
list_rateRIFTING=list() # rate (km/Myr calculated relative to start of rifting)

list_ids=list() # list of kimberlite IDs in each location (so we can check allocation etc.)

list_breakupages=list()

## keep tabs on the individual rift sections
## note: there are more 'sections' for breakup age (FROMAGE) than rift age as the rift unzips

breakup_sections=data.frame(Location=character(), BreakupAge=integer(), RiftAge=integer())


#################################################################################
#########################  Associate with ridge section #########################
### Loop through each continent ("Locations") and calculate Kimberlite distance 
### and lag from each rift section
#################################################################################
#################################################################################
### Evaluating kimberlite date (lag) relative to FROMAGE (breakup age)
### this is a generic function applicable to multiple rift sections in a single
### continental region. 
### For each rift section, we include all kimberlites with ages from 10 Myr
### before rift initiation (and any time after initiation).
###
### Where a Kimberlite could be associated with multiple rifts, we test two 
### allocation rules: (1) choose the rift with the shortest distance
### and (2) choose the shortest lag
### this is a simple method to select the most probable association
###
### for N Amercia/Canada we are focusing on a single, inititating section
### Note distance is converted to km so rate is Dist/Lag =km/Myr
#################################################################################

#################################################################################
### To accommodate larger distances/timescales for America increase distance limit
### also limit the rate to exclude extreme (and physically unrealistic) associations
### note this rate constraint is still very conservative as it allows very slow 
### (close to 0km/Myr) propagation

LIMIT_DIST=3000 
LIMIT_RATE=100 # determine association by rate (rel RIFTING) between 0 and 100 km/Myr
NUM_SAMPLES=5000 # number of samples for age uncertainty

set.seed(42) # set a random number seed if you want to make calcs repeatable

# list to contain all the results (dist/lag calculations for the different regions 
# and rifting episodes):
Results_all=list()

# kimberlite ages (all sampled values with uncertainty) by location
All_sampled_ages=list()

# loop through continents ("Locations"):
# note for Africa we consider multiple rift sections/events

#################################################################################
## LOOP THROUGH LOCATIONS 

for(p in 1:length(Locations)){

myrift=Rifts[p]
mylocation=Locations[p]

print(paste("Calculating time lags and distances for", myrift, ",", mylocation))

COB_0Ma <- st_read(dsn = "../GIS", layer = myrift)

# select Kimberlites in current region (Location):
Kimberlites=AllKimberlites[which(AllKimberlites$Location==mylocation),] 

list_ids[[mylocation]]=Kimberlites$fid

# add unique (integer) ID ridge section(s) 
COB_0Ma$fid<-1:nrow(COB_0Ma)

# find the shortest distance from each kimberlite to COB(s) 
# NOTE ages are in Ma, so increasing back in time
# Using the function dist2Line(p, line, distfun=distGeo)
# from package geosphere, where:
#
# p = longitude/latitude of point(s). Can be a vector of two numbers, a matrix of 2
# 	columns (first one is longitude, second is latitude) or a SpatialPoints* object
# line = longitude/latitude of line as a matrix of 2 columns (first one is longitude, second
# 	is latitude) or a SpatialLines* or SpatialPolygons* object
# distfun= A distance function, such as distGeo
# 	distHaversine: The shortest distance between two points (i.e., the ’great-circle-distance’ 
#	or ’as the crow flies’) according to the ’haversine method’. This method assumes a 
# 	spherical earth, ignoring ellipsoidal effects.

# convert points and lines to simple features:
Kimberlites_sp=sf::as_Spatial(Kimberlites)
COB_0Ma_sp=sf::as_Spatial(COB_0Ma)

# ages of the rift section(s) to step through:
breakup_age=unique(na.omit(COB_0Ma_sp$FROMAGE))


list_breakupages[[mylocation]]=breakup_age

numk=length(Kimberlites_sp)
#numl=length(COB_0Ma_sp) # note there are more sections than ages, as some have multiple parts
# we need to merge the ones with the same age

# list to contain the results for current location only:
# (this is then added to the Results_all list)
results_list=list()

# NUM_SAMPLES sampled values, plus the original quoted value for comparison as LAST COL
Kim_sampled_ages=matrix(nrow=numk,ncol=NUM_SAMPLES+1)

for(i in 1:numk) Kim_sampled_ages[i,] = c(runif(NUM_SAMPLES, 
	min=Kimberlites_sp$Age[i]-Kimberlites_sp$Uncert[i], 
	max=Kimberlites_sp$Age[i]+Kimberlites_sp$Uncert[i]) , Kimberlites_sp$Age[i]  )

All_sampled_ages[[mylocation]]=Kim_sampled_ages

#################################################################################
## LOOP THROUGH AGE SECTIONS 

for(i in 1:length(breakup_age)) {

# get line section with current loop rift onset age:
COBs= subset(COB_0Ma_sp, FROMAGE==breakup_age[i])

# allow the estimated distance from Rift to Kimberlite to vary (+/- 90 km )
# final value (zero) is for the original data (no simulated error added)
Dist_sampled_offset=c(runif(NUM_SAMPLES, min=-90, max=90),0) 
#

# temporary array for kimberlite-rift section distances:
# make 2D matrix as we have nrow=nl, ncol=NUM_SAMPLES+1 (for the sampled offset)
# the final column NUM_SAMPLES+1 is used to store the original (unvaried) estimate
temp_dists=matrix(nrow=numk,ncol=(NUM_SAMPLES+1) )

# loop through Kims$fid as not all kimberlite IDs are used
# USING ALL KIMBERLITES NOW - and cutting those outside the range of plausible lag/dist later
for(j in 1:numk) {
kpoint= subset(Kimberlites_sp, fid==Kimberlites$fid[j])
# NOTE converting the distance from m (returned by geosphere fn) to km here
temp_dists[j,]=(dist2Line(kpoint, COBs, distfun=distGeo)[1]/1e3)+Dist_sampled_offset # shortest dist between point and line WITH UNCERTAINTY OFFSET
	}

#############
# have matrix with distances (temp_dists) kim rows x samples cols
# now need matrix with lags

# vary breakup age by 5% (+/- 2.5%) uniform dist
temp_sampled_breakup=c(runif(NUM_SAMPLES, 
	min=0.975*breakup_age[i], 
	max=1.025*breakup_age[i]), breakup_age[i])

temp_sampled_lags=matrix(nrow=numk,ncol=(NUM_SAMPLES+1) )
for(j in 1:numk) temp_sampled_lags[j,]= temp_sampled_breakup- Kim_sampled_ages[j,]
# matrix of lags row:= kimberlite no, col:= sample no


# calculate the effective rate rel to RIFTING
# note this only works when there is a single breakup age (FROMAGE) associated with
# all the lines sections in the subset with a given RiftAge 
# - if there are more combinations then method needs modifying

print("************************************************")

print(paste("CHECK RIFT AND BREAKUP AGES",length(COBs$RiftAge), length(COBs$FROMAGE) ))

rift_breakup_offset=unique(COBs$RiftAge)-unique(COBs$FROMAGE)

print(paste("CHECK UNIQUE",length(rift_breakup_offset) ))

temp_rates_RIFTING=temp_dists/(rift_breakup_offset+temp_sampled_lags) 


name <- paste(mylocation,"_Breakup_age_",breakup_age[i],sep="") 
list_lag_breakup[[name]]=temp_sampled_lags
list_dist_breakup[[name]]=temp_dists
list_rateRIFTING[[name]]= temp_rates_RIFTING

# quick look:
#hist(temp_sampled_lags[which((temp_sampled_lags>-75)&(temp_sampled_lags<100))], main=name)
#hist(temp_dists[which((temp_sampled_lags>-75)&(temp_sampled_lags<100))], main=name)

print(name)
print ("Done breakup age calcs")
} #end i loop (BREAKUP ages)


} #end p loop (LOCATIONS)



#################################################################################
##############             PART 1: PERFORM ALLOCATIONS              #############
#################################################################################

#################################################################################
##############  CALCULATE LAG REL TO BREAKUP       
##############	for individual kimberlites                


MINLAG_sampled_breakup_lags=list()
MINLAG_sampled_breakup_dists=list()
MINLAG_sampled_breakup_indices=list()
MINLAG_sampled_breakup_rates=list()

MINDIST_sampled_breakup_lags=list()
MINDIST_sampled_breakup_dists=list()
MINDIST_sampled_breakup_indices=list()
MINDIST_sampled_breakup_rates=list()


for(p in 1:length(Locations)){

mylocation=Locations[p]
nkim=length(list_ids[[mylocation]])

MINLAG_sampled_breakup_lags[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINLAG_sampled_breakup_dists[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINLAG_sampled_breakup_indices[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINLAG_sampled_breakup_rates[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))

MINDIST_sampled_breakup_lags[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINDIST_sampled_breakup_dists[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINDIST_sampled_breakup_indices[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))
MINDIST_sampled_breakup_rates[[mylocation]]=matrix(nrow=nkim, ncol=(NUM_SAMPLES+1))

temp_names=paste(mylocation,"_Breakup_age_",list_breakupages[[mylocation]],sep="") 

for(sample_no in 1:(NUM_SAMPLES+1)){

temp_dist=list_dist_breakup[[temp_names[1]]][,sample_no]
for(i in 2:length(temp_names)) temp_dist=cbind(temp_dist, list_dist_breakup[[temp_names[i]]][,sample_no])

temp_lag=list_lag_breakup[[temp_names[1]]][,sample_no]
for(i in 2:length(temp_names)) temp_lag=cbind(temp_lag, list_lag_breakup[[temp_names[i]]][,sample_no])

temp_rate=list_rateRIFTING[[temp_names[1]]][,sample_no]
for(i in 2:length(temp_names)) temp_rate=cbind(temp_rate, list_rateRIFTING[[temp_names[i]]][,sample_no])

# impose limits for plausible rate and distance association:
temp_lag[which((temp_rate<0)|(temp_rate>LIMIT_RATE)|(temp_dist>LIMIT_DIST))]=NA
temp_rate[which(is.na(temp_lag))]=NA 
temp_dist[which(is.na(temp_lag))]=NA



############################################
# THIS TAKES THE MIN LAG

test=max.col(-replace(temp_lag, is.na(temp_lag), Inf), ties="first")
## NOTE we have some NA cols, correct for that next:
test=test* NA^!rowSums(!is.na(temp_lag))

rates=temp_rate[cbind(seq_along(test), test)]
lags=temp_lag[cbind(seq_along(test), test)]
dists=temp_dist[cbind(seq_along(test), test)]
# saves the selected rate and lags

MINLAG_sampled_breakup_lags[[mylocation]][,sample_no] = lags 
MINLAG_sampled_breakup_dists[[mylocation]][,sample_no] = dists 
MINLAG_sampled_breakup_rates[[mylocation]][,sample_no] = rates 
MINLAG_sampled_breakup_indices[[mylocation]][,sample_no] = test 


############################################
# THIS TAKES THE MIN DISTANCE

test=max.col(-replace(temp_dist, is.na(temp_dist), Inf), ties="first")
## NOTE we have some NA cols, correct for that next:
test=test* NA^!rowSums(!is.na(temp_dist))

rates=temp_rate[cbind(seq_along(test), test)]
lags=temp_lag[cbind(seq_along(test), test)]
dists=temp_dist[cbind(seq_along(test), test)]
# saves the selected rate and lags

MINDIST_sampled_breakup_lags[[mylocation]][,sample_no] = lags 
MINDIST_sampled_breakup_dists[[mylocation]][,sample_no] = dists 
MINDIST_sampled_breakup_rates[[mylocation]][,sample_no] = rates 
MINDIST_sampled_breakup_indices[[mylocation]][,sample_no] = test 


} # end sample loop

} # end loop p location


#################################################################################
#################################################################################
#################   PART 2:       CLUSTERED DATA                 ################
#################  GENERATE CLUSTER INDICES TO APPLY TO RESULTS  ################

# cluster points close together in space and time
# NOTE AllKimberlites includes clusters outside of the areas we are studying (Europe etc)
# could remove these first

# make a list for all the clusters
# each entry in the list has the indices of the points in that particular cluster
# starting with the first one i=1

cluster_indices=list()


sf_point_Kimb  <- st_as_sf(x = AllKimberlites, 
                        coords = c("Longitude", "Latitude"),
                        crs = "+proj=longlat +datum=WGS84 +ellps=WGS84 +towgs84=0,0,0")

sp_point_Kimb = as(sf_point_Kimb, "Spatial")

distance_matrix=distm(sp_point_Kimb, sp_point_Kimb, distGeo)
distance_matrix=distance_matrix/1000 
#distance now in km
num_k=length(AllKimberlites$fid)
dt_matrix=outer(AllKimberlites$Age,AllKimberlites$Age,"-")

# test for distance <=25km and lag<=5myr

cluster_matrix=matrix(ncol=num_k, nrow=num_k)

for(i in 1:num_k) {
	for (j in 1:num_k) {  # loop from 1 as although its slower, we need the full matrix for the method below

if((distance_matrix[i,j]<=25)&(abs(dt_matrix[i,j])<=5)) cluster_matrix[i,j]=1 
else cluster_matrix[i,j]=0 

	} #end col loop j
} # end row loop i



# keep track of which points have already been allocated to a cluster
point_allocated=rep(0, num_k)

 num_clusters=0

for(i in 1:num_k) {
x=NULL
if(!point_allocated[i]) {
	x=i
	point_allocated[i]=1
for (j in (i+1):num_k) {
if(i==num_k) break # exit if i =num_k (no more to add to cluster)
if(!point_allocated[j]){
	if(cluster_matrix[i,j]) { 
		x=c(x,j) 
		point_allocated[j]=1
	} else if (length(x)>1) { 
		for (p in 2:length(x)) if((!point_allocated[j])&(cluster_matrix[x[p],j])) {
			x=c(x,j)
			point_allocated[j]=1 
			}	# end if not allocated and clustered loop
		} # end if length(x)>1 loop
	# if point j isn't allocated and it is clustered with i, add it to the list
	# now check to see if j is clustered with any other points already in the list 
	#(may want to limit the overall size, but for now use simple rule of any association)

	} # end if point j NOT already allocated
} # end loop j

if(!is.null(x)){
 num_clusters= num_clusters+1 # if we have a cluster, add it to the list
cluster_indices[[num_clusters]]<-x }

}# end if point i NOT allocated 
} # end loop i

# NOTE kimberlites are AllKimberlites$fid[INDEX]
# when summing results over a cluster we need to eliminate locations outside study area FIRST 
# as we have no data for these points (dist, lag etc)

# can use an array as each cluster has a unique location 
# (in future can switch to list if clusters cross borders!)
cluster_locations=rep("NA",num_clusters)
for (i in 1: num_clusters) cluster_locations[i]= unique(AllKimberlites$Location[cluster_indices[[i]]])

# check everything is allocated to a cluster, and not duplicated:
length(unlist(cluster_indices))
length(unique(unlist(cluster_indices))) # should both be equal to the orig no. kimberlites

#################################################################################
#################################################################################
### get the cluster stats
### first loop through locations
###

# list (one entry per location) of cluster statistics

#list_cluster_breakup=list()
#list_breakup_lmcoeff=list()

MINDIST_list_cluster_breakup=list()
MINLAG_list_cluster_breakup=list()

first_cluster=TRUE
first_cluster2=TRUE
##################################################################
for(p in 1:length(Locations)){

mylocation=Locations[p]

print(paste("Calculating cluster summary data for", mylocation))

MINLAG_cluster_summary=data.frame(id=which(cluster_locations==mylocation), 
				av_dist=NA, av_lag=NA, 
				med_dist=NA, med_lag=NA,
				sd_dist=NA, sd_lag=NA,
				av_rate=NA, sd_rate=NA,
				av_age=NA, med_age=NA, sd_age=NA,
				location=mylocation)

MINDIST_cluster_summary=MINLAG_cluster_summary

nclusters=length(MINDIST_cluster_summary$id)


##################################################################
## calculate mean/sd lag and distance for RIFTING AGE 
## REPEAT THIS FOR THE MIN DIST AND MIN LAG ALLOCATION METHODS

for(i in 1:nclusters){ # step through all clusters in current location

mycluster=MINDIST_cluster_summary$id[i]
fids=cluster_indices[[mycluster]]
kimids=which(list_ids[[mylocation]]%in%fids)

##############################################
## USE MIN LAG METHOD:

templags=MINLAG_sampled_breakup_lags[[mylocation]][kimids,]
tempdists=MINLAG_sampled_breakup_dists[[mylocation]][kimids,]
temprates=MINLAG_sampled_breakup_rates[[mylocation]][kimids,]

if (first_cluster) {
	if(length(fids)>1){
		MINLAG_CLUSTER_sample_lags=matrix(colMeans(templags, na.rm=T), nrow=1)
		MINLAG_CLUSTER_sample_dists=matrix(colMeans(tempdists, na.rm=T), nrow=1)
		MINLAG_CLUSTER_sample_rates=matrix(colMeans(temprates, na.rm=T), nrow=1)
	}
	else {
		MINLAG_CLUSTER_sample_lags=matrix(templags, nrow=1)
		MINLAG_CLUSTER_sample_dists=matrix(tempdists, nrow=1)
		MINLAG_CLUSTER_sample_rates=matrix(temprates, nrow=1)
	}
first_cluster=FALSE
}
else {
	if(length(fids)>1){
		MINLAG_CLUSTER_sample_lags=rbind(MINLAG_CLUSTER_sample_lags,colMeans(templags, na.rm=T))
		MINLAG_CLUSTER_sample_dists=rbind(MINLAG_CLUSTER_sample_dists,colMeans(tempdists, na.rm=T))
		MINLAG_CLUSTER_sample_rates=rbind(MINLAG_CLUSTER_sample_rates,colMeans(temprates, na.rm=T))
	}
	else {
		MINLAG_CLUSTER_sample_lags=rbind(MINLAG_CLUSTER_sample_lags,templags)
		MINLAG_CLUSTER_sample_dists=rbind(MINLAG_CLUSTER_sample_dists,tempdists)
		MINLAG_CLUSTER_sample_rates=rbind(MINLAG_CLUSTER_sample_rates,temprates)

	} 
}


MINLAG_cluster_summary$av_lag[i]=mean(templags,na.rm=T)
MINLAG_cluster_summary$av_dist[i]=mean(tempdists,na.rm=T)
MINLAG_cluster_summary$sd_lag[i]=sd(templags,na.rm=T)
MINLAG_cluster_summary$sd_dist[i]=sd(tempdists,na.rm=T)

MINLAG_cluster_summary$med_lag[i]=median(templags,na.rm=T)
MINLAG_cluster_summary$med_dist[i]=median(tempdists,na.rm=T)
MINLAG_cluster_summary$med_rate[i]=median(tempdists,na.rm=T)

MINLAG_cluster_summary$av_rate[i]=mean(temprates,na.rm=T)
MINLAG_cluster_summary$sd_rate[i]=sd(temprates,na.rm=T)

## SD AGE INCORPORATING UNCERTAINTY SAMPLING
MINLAG_cluster_summary$av_age[i]=mean(All_sampled_ages[[mylocation]][kimids,])
MINLAG_cluster_summary$med_age[i]=median(All_sampled_ages[[mylocation]][kimids,])
MINLAG_cluster_summary$sd_age[i]=sd(All_sampled_ages[[mylocation]][kimids,])
# note 1:NUM_SAMPLES are the sampled values (cols) and NUM_SAMPLES+1 is the original unaltered value
#######################


##########################################################
#### USE MIN DIST METHOD:
templags=MINDIST_sampled_breakup_lags[[mylocation]][kimids,]
tempdists=MINDIST_sampled_breakup_dists[[mylocation]][kimids,]
temprates=MINDIST_sampled_breakup_rates[[mylocation]][kimids,]

if (first_cluster2) {
	if(length(fids)>1){
		MINDIST_CLUSTER_sample_lags=matrix(colMeans(templags, na.rm=T), nrow=1)
		MINDIST_CLUSTER_sample_dists=matrix(colMeans(tempdists, na.rm=T), nrow=1)
		MINDIST_CLUSTER_sample_rates=matrix(colMeans(temprates, na.rm=T), nrow=1)
	}
	else {
		MINDIST_CLUSTER_sample_lags=matrix(templags, nrow=1)
		MINDIST_CLUSTER_sample_dists=matrix(tempdists, nrow=1)
		MINDIST_CLUSTER_sample_rates=matrix(temprates, nrow=1)
	}
first_cluster2=FALSE
}
else {
	if(length(fids)>1){
		MINDIST_CLUSTER_sample_lags=rbind(MINDIST_CLUSTER_sample_lags,colMeans(templags, na.rm=T))
		MINDIST_CLUSTER_sample_dists=rbind(MINDIST_CLUSTER_sample_dists,colMeans(tempdists, na.rm=T))
		MINDIST_CLUSTER_sample_rates=rbind(MINDIST_CLUSTER_sample_rates,colMeans(temprates, na.rm=T))
	}
	else {
		MINDIST_CLUSTER_sample_lags=rbind(MINDIST_CLUSTER_sample_lags,templags)
		MINDIST_CLUSTER_sample_dists=rbind(MINDIST_CLUSTER_sample_dists,tempdists)
		MINDIST_CLUSTER_sample_rates=rbind(MINDIST_CLUSTER_sample_rates,temprates)

	} 
}


MINDIST_cluster_summary$av_lag[i]=mean(templags,na.rm=T)
MINDIST_cluster_summary$av_dist[i]=mean(tempdists,na.rm=T)
MINDIST_cluster_summary$sd_lag[i]=sd(templags,na.rm=T)
MINDIST_cluster_summary$sd_dist[i]=sd(tempdists,na.rm=T)

MINDIST_cluster_summary$med_lag[i]=median(templags,na.rm=T)
MINDIST_cluster_summary$med_dist[i]=median(tempdists,na.rm=T)
MINDIST_cluster_summary$med_rate[i]=median(tempdists,na.rm=T)

MINDIST_cluster_summary$av_rate[i]=mean(temprates,na.rm=T)
MINDIST_cluster_summary$sd_rate[i]=sd(temprates,na.rm=T)

## SD AGE INCORPORATING UNCERTAINTY SAMPLING
MINDIST_cluster_summary$av_age[i]=mean(All_sampled_ages[[mylocation]][kimids,]) # this is actually the same as the MINLAG data
MINDIST_cluster_summary$med_age[i]=median(All_sampled_ages[[mylocation]][kimids,])
MINDIST_cluster_summary$sd_age[i]=sd(All_sampled_ages[[mylocation]][kimids,])  

# as the clusters contain the same kimberlites
# only the estimate for lag/dist change slightly
								 
} #end cluster loop i

MINDIST_list_cluster_breakup[[mylocation]]=MINDIST_cluster_summary
MINLAG_list_cluster_breakup[[mylocation]]=MINLAG_cluster_summary

} # end location p loop

# get rid of useless row names in matrices 
dimnames(MINDIST_CLUSTER_sample_lags) <- NULL
dimnames(MINDIST_CLUSTER_sample_dists) <- NULL
dimnames(MINDIST_CLUSTER_sample_rates) <- NULL
dimnames(MINLAG_CLUSTER_sample_lags) <- NULL
dimnames(MINLAG_CLUSTER_sample_dists) <- NULL
dimnames(MINLAG_CLUSTER_sample_rates) <- NULL
# these matrices contain NUM_SAMPLES lag/rate/dist values for each cluster 
# (averaged over all kimberlites in the cluster for a single run at a time)
# so the size is NUM_SAMPLES (rows) x num clusters (cols), for clusters in all three locations


## arrange the cluster (samples) in a way that we can make density plots
## (a) by location and (b) by method
## USING ALL THE INDIVIDUAL MODEL SAMPLES not just averages/medians

ll=c(MINDIST_list_cluster_breakup[[1]]$location,MINDIST_list_cluster_breakup[[2]]$location,MINDIST_list_cluster_breakup[[3]]$location)

long_MINLAG_CLUSTER_sample_rates=data.frame(rate=as.vector(MINLAG_CLUSTER_sample_rates),method="MinLag",location=ll)

long_MINDIST_CLUSTER_sample_rates=data.frame(rate=as.vector(MINDIST_CLUSTER_sample_rates),method="MinDist",location=ll)

long_CLUSTER_sample_rates=rbind(long_MINLAG_CLUSTER_sample_rates,long_MINDIST_CLUSTER_sample_rates)

long_CLUSTER_sample_rates=na.omit(long_CLUSTER_sample_rates)

# output the (sampled) kimberlite cluster rate estimates:
write.table(long_CLUSTER_sample_rates, "CLUSTER_sample_rates_Fig1e_ExFig4a.csv", sep=",", row.names=F, quote=F)

## now plot densities 

md_median=median(long_CLUSTER_sample_rates$rate,na.rm=T)
md_mean=mean(long_CLUSTER_sample_rates$rate,na.rm=T)
md=ggplot() + 
  geom_density(data=long_CLUSTER_sample_rates, aes(x=rate, group=location, fill=location), alpha=0.5, adjust=2) + 
  xlab("Rate (km/Myr)") + ggtitle("Clustered data, with model uncertainty") +
  ylab("Density")  +  theme_classic() +
  geom_vline(xintercept=md_median, linetype="dashed", color = "red") +
  geom_vline(xintercept=md_mean, linetype="dashed", color = "grey30") +theme_classic()

# get the max density value from the plot to set text annotation position
ymax <- layer_scales(md)$y$range$range[2]
# add the median and mean value:
md=md+ annotate("text", x=md_median+5, y=ymax, hjust = 0, vjust = 1,  
  label=paste("median = ",sprintf("%.2f",md_median)), color="red")
md=md+ annotate("text", x=md_mean+5, y=ymax*.7, hjust = 0, vjust = 1,  
  label=paste("mean = ",sprintf("%.2f",md_mean)), color="grey30")

md


############### PLOT cluster data by location

median= median(long_CLUSTER_sample_rates$rate,na.rm=T)
mean= mean(long_CLUSTER_sample_rates$rate,na.rm=T)
sd=sd(long_CLUSTER_sample_rates$rate,na.rm=T)

median_SAm=median(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="S_America")])
median_NAm=median(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="N_America")])
median_Af=median(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="Africa")])

mean_SAm=mean(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="S_America")])
mean_NAm=mean(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="N_America")])
mean_Af=mean(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="Africa")])

qs=c(0.1,0.5,0.9)
Cquantile_SAm=quantile(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="S_America")], qs)
Cquantile_NAm=quantile(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="N_America")], qs)
Cquantile_Af=quantile(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$location=="Africa")], qs)

Cquantile_all=quantile(long_CLUSTER_sample_rates$rate, qs)

d2a=ggplot() + 
  geom_density(data=long_CLUSTER_sample_rates, aes(x=rate, group=location, fill=location), alpha=0.5, adjust=2) + 
  xlab("Rate (km/Myr)") + ggtitle("Clustered data, with associated model uncertainty") +
  ylab("Density") +   theme_classic()+
  geom_vline(xintercept=median, linetype="dashed", color = "black") #+  
 # geom_vline(xintercept=mean, linetype="dashed", color = "grey30") 

# get the max density value from the plot to set text annotation position
ymax <- layer_scales(d2a)$y$range$range[2]
# add the median value:
d2a=d2a+ annotate("text", x=median+1, y=ymax*.7, hjust = 0, vjust = 1,   angle = 90,
  label=paste("median = ",sprintf("%.2f",median)), color="black")
#d2a=d2a+ annotate("text", x=mean+1, y=ymax*.7, hjust = 0, vjust = 1,   angle = 90,
#  label=paste("mean = ",sprintf("%.2f",mean)), color="grey30") 
d2a

medianvals=c( paste("SA:",sprintf("%.2f",median_SAm)) ,
	paste("NA:",sprintf("%.2f",median_NAm)),
	paste("Af:",sprintf("%.2f",median_Af)) )

line.data <- data.frame(xintercept = c(median_SAm,median_NAm,median_Af), Median_values = medianvals,
  color = c("blue4","darkgreen", "darkred"), stringsAsFactors = FALSE)

d2b=d2a+  geom_vline(data=line.data, aes(xintercept = xintercept, color=Median_values), linetype="dashed")

d2b

pdf(paste0("Fig1e_Density_Rate_rel_Rift_CLUSTERS_byLOCATION_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
d2b
dev.off()

############### PLOT cluster data by method

median= median(long_CLUSTER_sample_rates$rate)
mean= mean(long_CLUSTER_sample_rates$rate)

median_minlag= median(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinLag")])
mean_minlag= mean(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinLag")])
median_mindist= median(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinDist")])
mean_mindist= mean(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinDist")])


Cquantile_minlag=quantile(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinLag")], qs)
Cquantile_mindist=quantile(long_CLUSTER_sample_rates$rate[which(long_CLUSTER_sample_rates$method=="MinDist")], qs)


d3=ggplot() + 
  geom_density(data=long_CLUSTER_sample_rates, aes(x=rate, group=method, fill=method), alpha=0.5, adjust=2) + 
  xlab("Rate (km/Myr)") + ggtitle("Clustered data, with associated model uncertainty") +
  ylab("Density") +  theme_classic() +
  geom_vline(xintercept=median_mindist, linetype="dashed", color = "red") +  
  geom_vline(xintercept=median_minlag, linetype="dashed", color = "blue") +
  geom_vline(xintercept=mean_mindist, linetype="dashed", color = "darkred") +  
  geom_vline(xintercept=mean_minlag, linetype="dashed", color = "blue4") 

# get the max density value from the plot to set text annotation position
ymax <- layer_scales(d3)$y$range$range[2]
# add the median value:
d3=d3+ annotate("text", x=median_mindist+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("median = ",sprintf("%.2f",median_mindist)), color="red")
d3=d3+ annotate("text", x=median_minlag+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("median = ",sprintf("%.2f",median_minlag)), color="blue")

d3=d3+ annotate("text", x=mean_mindist+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("mean = ",sprintf("%.2f",mean_mindist)), color="darkred")
d3=d3+ annotate("text", x=mean_minlag+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("mean = ",sprintf("%.2f",mean_minlag)), color="blue4")

d3

pdf(paste0("ExFig4a_Density_Rate_rel_Rift_CLUSTERS_byMETHOD_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
d3
dev.off()

############## 
## test - quick look at where the method (min lag/dist) doesn't change the attribution
#same_res=long_MINLAG_CLUSTER_sample_rates$rate[which(long_MINLAG_CLUSTER_sample_rates$rate==long_MINDIST_CLUSTER_sample_rates$rate)]
#minlag_res=long_MINLAG_CLUSTER_sample_rates$rate[which(long_MINLAG_CLUSTER_sample_rates$rate!=long_MINDIST_CLUSTER_sample_rates$rate)]
#mindist_res=long_MINDIST_CLUSTER_sample_rates$rate[which(long_MINLAG_CLUSTER_sample_rates$rate!=long_MINDIST_CLUSTER_sample_rates$rate)]
##############

#########################################################################################
#########################################################################################
# plot individual kimberlite data incorporating all known sources of uncertainty
# - age,distance and model (association) uncertainty
#
# take the large matrices of n(rows) kimberlites x m(cols) samples
# and convert to an array using as.vector(matrix)

temp1=data.frame(rates=as.vector(MINLAG_sampled_breakup_rates[["Africa"]]),location="Africa",method="MinLag")
temp2=data.frame(rates=as.vector(MINLAG_sampled_breakup_rates[["S_America"]]),location="S_America",method="MinLag")
temp3=data.frame(rates=as.vector(MINLAG_sampled_breakup_rates[["N_America"]]),location="N_America",method="MinLag")

kim_rates=rbind(temp1,temp2,temp3)
rm(temp1,temp2,temp3)
kim_rates=na.omit(kim_rates)

temp1=data.frame(rates=as.vector(MINDIST_sampled_breakup_rates[["Africa"]]),location="Africa",method="MinDist")
temp2=data.frame(rates=as.vector(MINDIST_sampled_breakup_rates[["S_America"]]),location="S_America",method="MinDist")
temp3=data.frame(rates=as.vector(MINDIST_sampled_breakup_rates[["N_America"]]),location="N_America",method="MinDist")

kim_rates=rbind(kim_rates,temp1,temp2,temp3)

kim_rates=na.omit(kim_rates)
# output the (sampled) individual kimberlite rate estimates if needed:
# NOTE THIS IS VERY LARGE!
# write.table(kim_rates, "KIMBERLITE_sample_rates_ExFig4b.csv", sep=",", row.names=F, quote=F)

####################################################################################
####################################################################################
########### plot by method


median_minlag= median(kim_rates$rates[which(kim_rates$method=="MinLag")])
mean_minlag= mean(kim_rates$rates[which(kim_rates$method=="MinLag")])
median_mindist= median(kim_rates$rates[which(kim_rates$method=="MinDist")])
mean_mindist= mean(kim_rates$rates[which(kim_rates$method=="MinDist")])

qs=c(0.1,0.5,0.9)
quantile_minlag=quantile(kim_rates$rates[which(kim_rates$method=="MinLag")], qs)
quantile_mindist=quantile(kim_rates$rates[which(kim_rates$method=="MinDist")], qs)
quantile_all=quantile(kim_rates$rates, qs)

k1=ggplot() + 
  geom_density(data=kim_rates, aes(x=rates, group=method, fill=method), alpha=0.5, adjust=2) + 
  xlab("Rate (km/Myr)") + ggtitle("Kimberlite data, with assoc model uncertainty") +
  ylab("Density") +    theme_classic()+
  geom_vline(xintercept=median_mindist, linetype="dashed", color = "red") +  
  geom_vline(xintercept=median_minlag, linetype="dashed", color = "blue") +
  geom_vline(xintercept=mean_mindist, linetype="dashed", color = "darkred") +  
  geom_vline(xintercept=mean_minlag, linetype="dashed", color = "blue4") 

# get the max density value from the plot to set text annotation position
ymax <- layer_scales(k1)$y$range$range[2]
# add the median value:
k1=k1+ annotate("text", x=median_mindist+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("median = ",sprintf("%.2f",median_mindist)), color="red")
k1=k1+ annotate("text", x=median_minlag+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("median = ",sprintf("%.2f",median_minlag)), color="blue")

k1=k1+ annotate("text", x=mean_mindist+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("mean = ",sprintf("%.2f",mean_mindist)), color="darkred")
k1=k1+ annotate("text", x=mean_minlag+1, y=ymax*.7, hjust = 0, vjust = 1,  angle = 90,
  label=paste("mean = ",sprintf("%.2f",mean_minlag)), color="blue4")

k1

pdf(paste0("ExFig4b_Density_Rate_rel_Rift_KIMBERLITES_byMETHOD_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
k1
dev.off()


####################################################################################
####################################################################################
############# plot by location


median= median(kim_rates$rates)
mean= mean(kim_rates$rates)
sd=sd(kim_rates$rates)

median_SAm=median(kim_rates$rates[which(kim_rates$location=="S_America")])
median_NAm=median(kim_rates$rates[which(kim_rates$location=="N_America")])
median_Af=median(kim_rates$rates[which(kim_rates$location=="Africa")])

quantile_SAm=quantile(kim_rates$rates[which(kim_rates$location=="S_America")], qs)
quantile_NAm=quantile(kim_rates$rates[which(kim_rates$location=="N_America")], qs)
quantile_Af=quantile(kim_rates$rates[which(kim_rates$location=="Africa")], qs)

k2a=ggplot() + 
  geom_density(data=kim_rates, aes(x=rates, group=location, fill=location), alpha=0.5, adjust=2) + 
  xlab("Rate (km/Myr)") + ggtitle("Kimberlite data, with assoc model uncertainty") +
  ylab("Density") +   theme_classic()+
  geom_vline(xintercept=median, linetype="dashed", color = "black") #+  
 # geom_vline(xintercept=mean, linetype="dashed", color = "grey30") 

# get the max density value from the plot to set text annotation position
ymax <- layer_scales(k2a)$y$range$range[2]
# add the median value:
k2a=k2a+ annotate("text", x=median+1, y=ymax*.7, hjust = 0, vjust = 1,   angle = 90,
  label=paste("median = ",sprintf("%.2f",median)), color="black")
#k2a=k2a+ annotate("text", x=mean+1, y=ymax*.7, hjust = 0, vjust = 1,   angle = 90,
#  label=paste("mean = ",sprintf("%.2f",mean)), color="grey30") 
#k2a

medianvals=c( paste("SA:",sprintf("%.2f",median_SAm)) ,
	paste("NA:",sprintf("%.2f",median_NAm)),
	paste("Af:",sprintf("%.2f",median_Af)) )

line.data <- data.frame(xintercept = c(median_SAm,median_NAm,median_Af), Median_values = medianvals,
  color = c("blue4","darkgreen", "darkred"), stringsAsFactors = FALSE)

k2b=k2a+  geom_vline(data=line.data, aes(xintercept = xintercept, color=Median_values), linetype="dashed")

k2b

pdf(paste0("UnusedAdditional_Density_Rate_rel_Rift_KIMBERLITES_byLOCATION_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
k2b
dev.off()


####################################################################################
####################################################################################
###                  PLOT HISTOGRAMS FOR NORTH AMERICA DATA                      ###

## INDIVIDUAL KIMBERLITES, WITH UNCERTAINTY 
subtitle="With time/dist uncertainty, min rift distance atribution method"
MinDist_N_America= data.frame(lag=na.omit(as.vector(MINDIST_sampled_breakup_lags[["N_America"]])))
h1=ggplot(data=MinDist_N_America,aes(x=lag)) + 
  geom_histogram(breaks=seq(-65,130,10), color="black",fill="grey80", aes(y=after_stat(count)/sum(after_stat(count)))) +
	ggtitle("Lag relative to breakup. Kimberlite data", subtitle=subtitle) +theme_classic()
h1

pdf(paste0("UnusedAdditional_LagHistogram_Kimberlites_NAmerica_MINDIST_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
h1
dev.off()

## KIMBERLITE CLUSTERS, WITH UNCERTAINTY 
ll=c(MINDIST_list_cluster_breakup[[1]]$location,MINDIST_list_cluster_breakup[[2]]$location,MINDIST_list_cluster_breakup[[3]]$location)
long_MINDIST_CLUSTER_sample_lags=na.omit( data.frame(lag=as.vector(MINDIST_CLUSTER_sample_lags),method="MinDist",location=ll) )

MinDist_N_America_CLUSTER= data.frame(lag=long_MINDIST_CLUSTER_sample_lags$lag[which(long_MINDIST_CLUSTER_sample_lags$location=="N_America")] )

h2=ggplot(data=MinDist_N_America_CLUSTER,aes(x=lag)) + 
  geom_histogram(breaks=seq(-65,130,10), color="black", fill="grey60", aes(y=after_stat(count)/sum(after_stat(count)))) +
	ggtitle("Lag relative to breakup. Cluster data", subtitle=subtitle) +theme_classic()
h2

pdf(paste0("ExFig5b_LagHistogram_KimberliteCLUSTERS_NAmerica_MINDIST_",NUM_SAMPLES,"samples.pdf"), width = 7, height = 5.5)
h2
dev.off()

####################################################################################
####################################################################################

## plot again with original kimberlite data, no time/dist uncertainty
## individual kimberlites, no uncertainty

temp=MINDIST_sampled_breakup_lags[["N_America"]][,(NUM_SAMPLES+1)]
temp=na.omit(temp)

subtitle=paste0("Total kimberlites: ", sum(!is.na(temp)), ". No uncertainty, min rift distance attribution" )

MinDist_N_America= data.frame(lag=temp)
h3=ggplot(data=MinDist_N_America,aes(x=lag)) + 
  geom_histogram(breaks=seq(-65,130,10), color="black",fill="grey90") +coord_cartesian(ylim = c(0,30))+
	ggtitle("Lag relative to breakup, Kimberlite data", subtitle=subtitle ) +
	theme_classic()
h3

pdf("UnusedAdditional_LagHistogram_Kimberlites_NAmerica_MINDIST_noUncertainty.pdf", width = 7, height = 5.5)
h3
dev.off()
###########

# north america is location 1
n= length(MINDIST_list_cluster_breakup[[1]]$location) 

temp=MINDIST_CLUSTER_sample_lags[1:n, (NUM_SAMPLES+1) ] # this is the unvaried data for N America only
temp=na.omit(temp)
subtitle=paste0("Total clusters: ", sum(!is.na(temp)), ". No uncertainty, min rift distance attribution"  )

MinDist_N_America_CLUSTER= data.frame(lag=temp )

h4=ggplot(data=MinDist_N_America_CLUSTER,aes(x=lag)) + 
  geom_histogram(breaks=seq(-65,130,10), color="black", fill="grey60") +coord_cartesian(ylim = c(0,10)) +
	ggtitle("Lag relative to breakup, Cluster data", subtitle=subtitle ) +
	theme_classic()
h4
pdf("ExFig5a_LagHistogram_KimberliteCLUSTERS_NAmerica_MINDIST_noUncertainty.pdf", width = 7, height = 5.5)
h4
dev.off()



